import path from 'path';
import fs from 'fs';
import ora from 'ora';
import install from 'spawn-npm-install';
import { ncp } from 'ncp';
import temp from 'temp';

import {
  exit, mkdir, cd, exec,
} from 'shelljs';

import logger from './logger';

import {
  resolvePath,
} from './utils';

import {
  TEMPLATE_REPO,
  DEPENDENCIES,
  PACKAGE_JSON,
} from './config';

export function createProjectSync(projectDir) {
  const spinner = ora(`1. Create project in ${projectDir}`).start();

  try {
    const mkdirResult = mkdir(projectDir);

    if (mkdirResult.code !== 0) {
      throw new Error(mkdirResult.stderr);
    }

    const cdResult = cd(projectDir);

    if (cdResult.code !== 0) {
      throw new Error(cdResult.stderr);
    }

    spinner.succeed();
  } catch (error) {
    logger.error(error.message);
    spinner.fail();
    exit(1);
  }
}

export function configPackageSync() {
  const spinner = ora('2. Configurate package.json').start();

  try {
    exec('npm init --force', { silent: true });

    /* eslint-disable global-require */
    const packageJson = require(resolvePath('package.json'));
    /* eslint-enable */

    const overridedConfig = Object.assign(packageJson, PACKAGE_JSON);
    const stringifiedConfig = JSON.stringify(overridedConfig, null, 2);

    fs.writeFileSync(resolvePath('package.json'), stringifiedConfig);

    spinner.succeed();
  } catch (error) {
    logger.error(error.message);
    spinner.fail();
    exit(1);
  }
}

export function copyTemplates(projectDir) {
  return new Promise((resolve, reject) => {
    const spinner = ora('3. Copy template files to src folder').start();

    const tempPath = temp.mkdirSync('templates');

    exec(`git clone --quiet ${TEMPLATE_REPO} ${tempPath}`);

    const source = path.join(tempPath, 'templates');
    const destination = projectDir;

    ncp(source, destination, error => {
      if (error) {
        logger.error(error);
        spinner.fail();
        reject(error);
        return;
      }

      spinner.succeed();
      resolve();
    });
  });
}

export function installPackage() {
  const spinner = ora('4. Install package dependencies').start();

  const depPromises = DEPENDENCIES.map(depSettings => (
    new Promise((resolve, reject) => {
      const persistOption = depSettings.dev ? { saveDev: true } : { save: true };

      install(
        depSettings.packages,
        Object.assign(persistOption, { stdio: 'inherit' }),
        error => {
          if (error) {
            reject(error.message);
            return;
          }

          resolve();
        }
      );
    })
  ));

  return Promise
    .all(depPromises)
    .then(result => {
      spinner.succeed();
      return result;
    })
    .catch(errors => {
      spinner.fail();
      throw errors;
    });
}

export function versionControlSync() {
  const commands = [
    'git init',
    'git add .',
    'git commit -m "Initial commit"',
  ];

  const spinner = ora('5. Setup version control').start();

  try {
    commands.forEach(command => {
      const result = exec(command, { silent: true });

      if (result.code !== 0) {
        throw new Error(result.stderr);
      }
    });

    spinner.succeed();
  } catch (error) {
    logger.error(error.message);
    spinner.fail();
    exit(1);
  }
}
